/*

NSAttributedString-NIRAdditionsTest.m

DirectionService - controll writting direction
Copyright (C) 2003 Nir Soffer

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details: http://www.gnu.org/licenses/gpl.html

*/

#import "NSAttributedString-NIRAdditionsTest.h"
#import "NSAttributedString-NIRAdditions.h"
#import "NSString-NIRAdditions.h"


@implementation NSAttributedStringNIRAdditionsTest

- (void)setUp
{
    // Create attributed string with different attributs on each line
    orgText = [[NSMutableAttributedString alloc] initWithString:
        @"line 1\nline 2\n\nLine 4"];
    [orgText applyFontTraits:NSBoldFontMask range:NSMakeRange(0,7)];
    [orgText setAlignment:NSCenterTextAlignment range:NSMakeRange(7,7)];
    [orgText addAttribute:NSForegroundColorAttributeName
                    value:[NSColor redColor]
                      range:NSMakeRange(15,6)];
    [orgText fixAttributesInRange:NSMakeRange(0, [orgText length])];

    // And make a copy for testing
    newText = [orgText mutableCopy];
}

- (void)tearDown
{
    [orgText release];
    [newText release];
}


- (void)testEnclose
{
    NSRange range;
    NSString *start = @"START";
    NSString *end = @"END";

    [newText encloseWithStart:start end:end fixAttributes:YES];

    // should start with start string
    range = NSMakeRange(0, [start length]);
    [self assert:[[newText mutableString] substringWithRange:range]
          equals:start];
    // with the attributes of the next character after start
    [self assert:[newText attributesAtIndex:[start length]
                          effectiveRange:NULL]
          equals:[newText attributesAtIndex:[start length] + 1
                          effectiveRange:NULL]];
    
    // should end with end string
    range = NSMakeRange([newText length] - [end length], [end length]);
    [self assert:[[newText mutableString] substringWithRange:range]
          equals:end];

    // with the attributes of the character before end
    [self assert:[newText attributesAtIndex:[start length] - [end length]
                          effectiveRange:NULL]
          equals:[newText attributesAtIndex:[start length] - [end length] - 1
                          effectiveRange:NULL]];
    
    // between them - the original attributed string
    range = NSMakeRange([start length],
                        [newText length] - [end length] - [start length]);
    [self assert:[newText attributedSubstringFromRange:range]
          equals:orgText];
}

- (void)testPrefixLines
{
    unsigned orgLocation, newLocation;
    unsigned length;
    unsigned orgStart, newStart;
    NSRange range;
    NSString *prefix = @"PREFIX";

    [newText prefixLinesWithString:prefix fixAttributes:YES];

    // iterate over the lines of both original text and the new text
    orgLocation = 0;
    newLocation = 0;

    while (orgLocation < [orgText length])
    {
        // get line start of original and new newText
        [[orgText mutableString] getLineStart:&orgStart
                                          end:&orgLocation
                                  contentsEnd:nil
                                     forRange:NSMakeRange(orgLocation,1)];
        [[newText mutableString] getLineStart:&newStart
                                          end:&newLocation
                                  contentsEnd:nil
                                     forRange:NSMakeRange(newLocation,1)];

        // prefix should be found here
        range = NSMakeRange(newStart, [prefix length]);
        [self assert:[[newText mutableString] substringWithRange:range]
              equals:prefix];

        // prefix should have the same attributes of the first character of
        // the original string. This can fail on some situations: before
        // attachment, ilegal font etc.
        [self assert:[newText attributesAtIndex:newStart
                                 effectiveRange:NULL]
              equals:[orgText attributesAtIndex:orgStart
                                 effectiveRange:NULL]];
        
        // After perfix should be the original line
        length = orgLocation - 1 - orgStart;
        [self assert:[orgText attributedSubstringFromRange:
            NSMakeRange(orgStart, length)]
              equals:[newText attributedSubstringFromRange:
                  NSMakeRange((newStart + [prefix length]), length)]];
    }
}

- (void)testDeleteCharactersFromSet
{
    // Todo: build a bigger, more genral test?

    NSCharacterSet *testSet = [NSCharacterSet characterSetWithRange:NSMakeRange(0x202A,3)];

    // Verify that string is clean
    NSRange range = [[orgText mutableString] rangeOfCharacterFromSet:testSet];
    [self assertInt:range.location
             equals:NSNotFound
            message:@"Test String is dirty"];

    // clean string should return the same
    [newText deleteCharactersFromSet:testSet fixAttributes:YES];
    [self assert:newText equals:orgText];

    // Insert characters from testSet - but not at the start of lines, because
    //this will break the paragraph style
    [[newText mutableString] insertString:[NSString stringWithCharacter:0x202A]
                                  atIndex:0];
    [[newText mutableString] insertString:[NSString stringWithCharacter:0x202B]
                                  atIndex:2];
    [[newText mutableString] insertString:[NSString stringWithCharacter:0x202C]
                                  atIndex:[newText length]];
    [newText fixAttributesInRange:NSMakeRange(0, [newText length])];

    // dirty string should return clean
    [newText deleteCharactersFromSet:testSet fixAttributes:YES];
    [self assert:newText equals:orgText];
}

@end
